Type layer
  Field ascii
  Field bank1
  Field bank2
  Field bank3
  Field code
  Field depth1
  Field depth2
  Field frame
  Field layer.layer
  Field mask
  Field mode
  Field name$
  Field parax
  Field paray
  Field posx
  Field posy
  Field sizex
  Field sizey
  Field start
  Field tile.tile
  Field time
  Field tmp
  Field visible
End Type

Const layer_back  =00
Const layer_map   =01
Const layer_iso1  =02
Const layer_iso2  =03
Const layer_hex1  =04
Const layer_hex2  =05
Const layer_clone =06
Const layer_image =07
Const layer_block =08
Const layer_point =09
Const layer_line  =10
Const layer_rect  =11
Const layer_oval  =12

Global layer_x
Global layer_y
Global layer_width
Global layer_height





;---------------------------------------------------------------------
;layer: layer handle
;---------------------------------------------------------------------
Function layer_delete(layer.layer)
  If layer=Null Then Return

  If layer\code<>layer_clone Then
    If layer\bank1<>0 Then FreeBank layer\bank1
    If layer\bank2<>0 Then FreeBank layer\bank2
  EndIf

  If layer\bank3<>0 Then FreeBank layer\bank3

  Delete layer
End Function





;---------------------------------------------------------------------
;layer:  layer handle
;x:      tile pos x
;y:      tile pos y
;RETURN: 0=void, 1=frame, 2=anim
;---------------------------------------------------------------------
Function layer_getcode(layer.layer,x,y)
  Local anims
  Local count
  Local mode
  Local offset
  Local value

  If layer=Null Then Return
  If layer\bank1=0 Then Return
  If x<0 Or y<0 Or x>layer\sizex-1 Or y>layer\sizey-1 Then Return

  If layer\tile<>Null Then
    anims=layer\tile\anims
    count=layer\tile\count
  EndIf

  If layer\depth1=4 Then
    offset=((y*layer\sizex+x)/2)
    mode  =((y*layer\sizex+x) And 1)*4
    value =(PeekByte(layer\bank1,offset) Shr mode) And 15
  ElseIf layer\depth1=8 Then
    offset=y*layer\sizex+x
    value=PeekByte(layer\bank1,offset)
  ElseIf layer\depth1=12 Then
    offset=((y*layer\sizex+x)*3)/2
    mode  =((y*layer\sizex+x) And 1)*4
    value =(PeekShort(layer\bank1,offset) Shr mode) And 4095
  ElseIf layer\depth1=16 Then
    offset=(y*layer\sizex+x)*2
    value=PeekShort(layer\bank1,offset)
  EndIf

  If value=>1 And value<=anims Then ;anim
    Return 2
  ElseIf value=>anims+1 And value<=count+anims Then ;frame
    Return 1
  EndIf
End Function





;---------------------------------------------------------------------
;layer:  layer handle
;x:      tile pos x
;y:      tile pos y
;RETURN: tile value
;---------------------------------------------------------------------
Function layer_getdata(layer.layer,x,y)
  Local mode
  Local offset
  Local value

  If layer=Null Then Return
  If layer\bank2=0 Then Return
  If x<0 Or y<0 Or x>layer\sizex-1 Or y>layer\sizey-1 Then Return

  If layer\depth2=4 Then
    offset=((y*layer\sizex+x)/2)
    mode  =((y*layer\sizex+x) And 1)*4
    value =(PeekByte(layer\bank2,offset) Shr mode) And 15
  ElseIf layer\depth2=8 Then
    offset=y*layer\sizex+x
    value =PeekByte(layer\bank2,offset)
  EndIf

  Return value
End Function





;---------------------------------------------------------------------
;layer:  layer handle
;x:      tile pos x
;y:      tile pos y
;RETURN: tile value
;---------------------------------------------------------------------
Function layer_getvalue(layer.layer,x,y)
  Local anims
  Local count
  Local mode
  Local offset
  Local value

  If layer=Null Then Return
  If layer\bank1=0 Then Return
  If x<0 Or y<0 Or x>layer\sizex-1 Or y>layer\sizey-1 Then Return

  If layer\tile<>Null Then
    anims=layer\tile\anims
    count=layer\tile\count
  EndIf

  If layer\depth1=4 Then
    offset=((y*layer\sizex+x)/2)
    mode  =((y*layer\sizex+x) And 1)*4
    value =(PeekByte(layer\bank1,offset) Shr mode) And 15
  ElseIf layer\depth1=8 Then
    offset=y*layer\sizex+x
    value=PeekByte(layer\bank1,offset)
  ElseIf layer\depth1=12 Then
    offset=((y*layer\sizex+x)*3)/2
    mode  =((y*layer\sizex+x) And 1)*4
    value =(PeekShort(layer\bank1,offset) Shr mode) And 4095
  ElseIf layer\depth1=16 Then
    offset=(y*layer\sizex+x)*2
    value=PeekShort(layer\bank1,offset)
  EndIf

  If value=>1 And value<=anims Then ;anim
    Return value
  ElseIf value=>anims+1 And value<=count+anims Then ;frame
    Return value-anims
  EndIf
End Function





;---------------------------------------------------------------------
;layer:  layer handle
;x:      tile pos x
;y:      tile pos y
;RETURN: raw hartcoded value
;---------------------------------------------------------------------
Function layer_getvalue2(layer.layer,x,y)
  Local mode
  Local offset
  Local value

  If layer=Null Then Return
  If layer\bank1=0 Then Return
  If x<0 Or y<0 Or x>layer\sizex-1 Or y>layer\sizey-1 Then Return

  If layer\depth1=4 Then
    offset=((y*layer\sizex+x)/2)
    mode  =((y*layer\sizex+x) And 1)*4
    value =(PeekByte(layer\bank1,offset) Shr mode) And 15
  ElseIf layer\depth1=8 Then
    offset=y*layer\sizex+x
    value=PeekByte(layer\bank1,offset)
  ElseIf layer\depth1=12 Then
    offset=((y*layer\sizex+x)*3)/2
    mode  =((y*layer\sizex+x) And 1)*4
    value =(PeekShort(layer\bank1,offset) Shr mode) And 4095
  ElseIf layer\depth1=16 Then
    offset=(y*layer\sizex+x)*2
    value=PeekShort(layer\bank1,offset)
  EndIf

  Return value
End Function





;---------------------------------------------------------------------
Function layer_reset()
  Local layer.layer

  For layer=Each layer
    If layer\code<>layer_clone Then
      If layer\bank1<>0 Then FreeBank layer\bank1
      If layer\bank2<>0 Then FreeBank layer\bank2
    EndIf
    If layer\bank3<>0 Then FreeBank layer\bank3
    Delete layer
  Next
End Function





;---------------------------------------------------------------------
;layer: layer handle
;x:     tile pos x
;y:     tile pos y
;value: tile value
;---------------------------------------------------------------------
Function layer_setdata(layer.layer,x,y,value)
  Local mode
  Local offset
  Local value2

  If layer=Null Then Return
  If layer\bank2=0 Then Return
  If x<0 Or y<0 Or x>layer\sizex-1 Or y>layer\sizey-1 Then Return
  If value<0 Then value=0

  If layer\depth2=4 Then
    If value>15 Then value=0
    offset=((y*layer\sizex+x)/2)
    mode  =((y*layer\sizex+x) And 1)*4
    value2=PeekByte(layer\bank2,offset) And ($F0 Shr mode)
    PokeByte layer\bank2,offset,value2 Or (value Shl mode)
  ElseIf layer\depth2=8 Then
    If value>255 Then value=0
    offset=y*layer\sizex+x
    PokeByte layer\bank2,offset,value
  EndIf
End Function





;---------------------------------------------------------------------
;layer: layer handle
;x:     tile pos x
;y:     tile pos y
;value: tile value
;code:  0=void, 1=frame, 2=anim
;---------------------------------------------------------------------
Function layer_setvalue(layer.layer,x,y,value,code)
  Local anims
  Local count
  Local mode
  Local offset
  Local value2

  If layer=Null Then Return
  If layer\bank1=0 Then Return
  If x<0 Or y<0 Or x>layer\sizex-1 Or y>layer\sizey-1 Then Return

  If layer\tile<>Null Then
    anims=layer\tile\anims
    count=layer\tile\count
  EndIf

  If code=1 Then
    If value<1 Or value>count Then value=0
    If value>0 Then value=value+anims
  ElseIf code=2 Then
    If value<1 Or value>anims Then value=0
  Else
    value=0
  EndIf

  If layer\depth1=4 Then
    If value>15 Then value=0
    offset=((y*layer\sizex+x)/2)
    mode  =((y*layer\sizex+x) And 1)*4
    value2=PeekByte(layer\bank1,offset) And ($F0 Shr mode)
    PokeByte layer\bank1,offset,value2 Or (value Shl mode)
  ElseIf layer\depth1=8 Then
    If value>255 Then value=0
    offset=y*layer\sizex+x
    PokeByte layer\bank1,offset,value
  ElseIf layer\depth1=12 Then
    If value>4095 Then value=0
    offset=((y*layer\sizex+x)*3)/2
    mode  =((y*layer\sizex+x) And 1)
    value2=PeekShort(layer\bank1,offset) And ($F000 Shr mode*12)
    PokeShort layer\bank1,offset,value2 Or (value Shl mode*4)
  ElseIf layer\depth1=16 Then
    If value>65535 Then value=0
    offset=(y*layer\sizex+x)*2
    PokeShort layer\bank1,offset,value
  EndIf
End Function





;---------------------------------------------------------------------
;layer: layer handle
;x:     tile pos x
;y:     tile pos y
;value: raw hardcoded value
;---------------------------------------------------------------------
Function layer_setvalue2(layer.layer,x,y,value)
  Local anims
  Local count
  Local mode
  Local offset
  Local value2

  If layer=Null Then Return
  If layer\bank1=0 Then Return
  If x<0 Or y<0 Or x>layer\sizex-1 Or y>layer\sizey-1 Then Return
  If value<0 Then value=0

  If layer\depth1=4 Then
    If value>15 Then value=0
    offset=((y*layer\sizex+x)/2)
    mode  =((y*layer\sizex+x) And 1)*4
    value2=PeekByte(layer\bank1,offset) And ($F0 Shr mode)
    PokeByte layer\bank1,offset,value2 Or (value Shl mode)
  ElseIf layer\depth1=8 Then
    If value>255 Then value=0
    offset=y*layer\sizex+x
    PokeByte layer\bank1,offset,value
  ElseIf layer\depth1=12 Then
    If value>4095 Then value=0
    offset=((y*layer\sizex+x)*3)/2
    mode  =((y*layer\sizex+x) And 1)
    value2=PeekShort(layer\bank1,offset) And ($F000 Shr mode*12)
    PokeShort layer\bank1,offset,value2 Or (value Shl mode*4)
  ElseIf layer\depth1=16 Then
    If value>65535 Then value=0
    offset=(y*layer\sizex+x)*2
    PokeShort layer\bank1,offset,value
  EndIf
End Function