;---------------------------------------------------------------------ANIMBANK
;offset byte description
;---------------------------------------------------------------------
;00     2    animation frames
;02     2    animation start
;04     2    animation mode (1=paused, 2=forward, 3=backward)
;06     4    last millisecs time
;10     2    current frame number
;12...  2    animation image
;14...  2    animation time
;---------------------------------------------------------------------

Type tile
  Field anims
  Field bank
  Field count
  Field factor
  Field file$
  Field image
  Field mask
  Field sizex
  Field sizey
End Type

Const tile_default=20
Const tile_minsize=08





;---------------------------------------------------------------------
Function tile_animate()
  Local bank
  Local frame
  Local frames
  Local i
  Local layer.layer
  Local mode
  Local tile.tile
  Local time1
  Local timediff
  Local timemax

  time2=MilliSecs()

  For tile=Each tile
    For i=1 To tile\anims

      bank    =PeekInt  (tile\bank,i*4-4)
      frames  =PeekShort(bank,0)
      mode    =PeekShort(bank,4)
      time1   =PeekInt  (bank,6)
      frame   =PeekShort(bank,10)
      timediff=time2-time1

      Repeat
        timemax=PeekShort(bank,frame*4+10)

        If mode=1 Or mode<0 Or mode>3 Or timemax=0 Then ;PAUSE
          PokeInt bank,6,time2
          Exit
        EndIf

        If timediff<=timemax Then
          PokeInt   bank,6 ,time2-timediff
          PokeShort bank,10,frame
          Exit
        EndIf

        timediff=timediff-timemax

        If mode=2 Then frame=frame+1 ;FORWARD
        If mode=3 Then frame=frame-1 ;BACKWARD

        If frame<1 Then frame=frames
        If frame>frames Then frame=1
      Forever

    Next
  Next


  For layer=Each layer
    If (layer\code=layer_image Or layer\code=layer_block) And layer\tile<>Null Then
      If layer\frame=>1 And layer\frame<=layer\tile\anims And layer\mode>0 Then

        bank    =PeekInt  (layer\tile\bank,layer\frame*4-4)
        frames  =PeekShort(bank,0)
        mode    =layer\mode
        time1   =layer\time
        frame   =layer\tmp
        timediff=time2-time1

        Repeat
          timemax=PeekShort(bank,frame*4+10)

          If mode=1 Or mode<0 Or mode>3 Or timemax=0 Then ;PAUSE
            layer\time=time2
            Exit
          EndIf

          If timediff<=timemax Then
            layer\time=time2-timediff
            layer\tmp=frame
            Exit
          EndIf

          timediff=timediff-timemax

          If mode=2 Then frame=frame+1 ;FORWARD
          If mode=3 Then frame=frame-1 ;BACKWARD

          If frame<1 Then frame=frames
          If frame>frames Then frame=1
        Forever

      EndIf
    EndIf
  Next
End Function





;---------------------------------------------------------------------
Function tile_animreset()
  Local bank
  Local i
  Local layer.layer
  Local start
  Local tile.tile

  time=MilliSecs()

  For tile=Each tile
    For i=1 To tile\anims
      bank=PeekInt(tile\bank,i*4-4)
      start=PeekShort(bank,2)
      PokeInt bank,6,time
      PokeShort bank,10,start
    Next
  Next

  For layer=Each layer
    If layer\code=layer_image Or layer\code=layer_block Then
      layer\time=time
      layer\tmp=layer\start
      If layer\tmp<0 Then layer\tmp=1
    EndIf
  Next
End Function





;---------------------------------------------------------------------
;RETURN: number of images with loading errors
;---------------------------------------------------------------------
Function tile_load()
  Local b
  Local error
  Local g
  Local r
  Local tile.tile

  For tile=Each tile
    If tile\image<>0 Then
      FreeImage tile\image
      tile\image=0
    EndIf

    If tile\file$<>"" Then
      tile\image=LoadAnimImage(map_path+tile\file$,tile\sizex,tile\sizey,0,tile\count)
      If tile\image=0 Then error=error+1
    EndIf

    If tile\image<>0 Then
      r=(tile\mask And $FF0000)/$10000
      g=(tile\mask And $FF00)/$100
      b=(tile\mask And $FF)
      MaskImage tile\image,r,g,b
    EndIf
  Next

  If map_image<>0 Then
    FreeImage map_image
    map_image=0
  EndIf

  If map_backgr$<>"" Then
    map_image=LoadImage(map_path$+map_backgr$)
    If map_image=0 Then error=error+1
  EndIf

  Return error
End Function





;---------------------------------------------------------------------
Function tile_reset()
  Local tile.tile

  For tile=Each tile
    If tile\bank <>0 Then FreeBank  tile\bank
    If tile\image<>0 Then FreeImage tile\image
    Delete tile
  Next
End Function